/*
 * ANALOG.cpp
 *
 *  Created on: 07/10/2010
 *      Author: mindurain
 */

#include "ANALOG.h"


void analogRead_gyro(analog* p_analog);
void analogRead_prox(analog* p_analog);


enum{
	ESTADO_ADC_REPOSO=0,
	ESTADO_ADC_INICIANDO_CONVERSION_X,
	ESTADO_ADC_ESPERANDO_FIN_CONVERSION_X,
	ESTADO_ADC_INICIANDO_CONVERSION_Y,
	ESTADO_ADC_ESPERANDO_FIN_CONVERSION_Y,
	ESTADO_ADC_INICIANDO_CONVERSION_Z,
	ESTADO_ADC_ESPERANDO_FIN_CONVERSION_Z,
};

#define PUERTO_X	0x01
#define PUERTO_Y	0x02
#define PUERTO_Z	0x00
#define PUERTO_PROX	0x03

void analg(analog* p_analog)
{
	switch(p_analog->estado)
	{
		case ESTADO_ANALOG_REPOSO:
		{
			//se mete el timer para cambiar el estado reposo
		}
		break;

		case ESTADO_ANALOG_LEYENDO_GYRO:
		{
			analogRead_gyro(p_analog);
		}
		break;

		case ESTADO_ANALOG_LEYENDO_PROXIMIDAD:
		{
			analogRead_prox(p_analog);
		}
		break;

		case ESTADO_ANALOG_MUESTREO_TERMINADO:
		{

		}
		break;
	}
}





void analogRead_gyro(analog* p_analog)
{
	uint8_t low, high;

	switch (p_analog->estado_sensor)
	{
		case ESTADO_ADC_REPOSO:
		{
			p_analog->estado_sensor=ESTADO_ADC_INICIANDO_CONVERSION_X;
		}
		break;

		case ESTADO_ADC_INICIANDO_CONVERSION_X:
		{
			// start the conversion
			ADMUX = 0b01000000 | (PUERTO_X & 0x07);
			sbi(ADCSRA, ADSC);
			p_analog->estado_sensor=ESTADO_ADC_ESPERANDO_FIN_CONVERSION_X;
		}
		break;

		case ESTADO_ADC_ESPERANDO_FIN_CONVERSION_X:
		{
			// ADSC is cleared when the conversion finishes
			if (bit_is_set(ADCSRA, ADSC)==0)
			{

				// we have to read ADCL first; doing so locks both ADCL
				// and ADCH until ADCH is read.  reading ADCL second would
				// cause the results of each conversion to be discarded,
				// as ADCL and ADCH would be locked when it completed.
				low = ADCL;
				high = ADCH;

				// combine the two bytes
				p_analog->x=(high << 8) | low;
				p_analog->estado_sensor=ESTADO_ADC_INICIANDO_CONVERSION_Y;
			}
		}
		break;

		case ESTADO_ADC_INICIANDO_CONVERSION_Y:
		{
			// start the conversion
			ADMUX = 0b01000000 | (PUERTO_Y & 0x07);
			sbi(ADCSRA, ADSC);
			p_analog->estado_sensor=ESTADO_ADC_ESPERANDO_FIN_CONVERSION_Y;
		}
		break;

		case ESTADO_ADC_ESPERANDO_FIN_CONVERSION_Y:
		{
			// ADSC is cleared when the conversion finishes
			if (bit_is_set(ADCSRA, ADSC)==0)
			{

				// we have to read ADCL first; doing so locks both ADCL
				// and ADCH until ADCH is read.  reading ADCL second would
				// cause the results of each conversion to be discarded,
				// as ADCL and ADCH would be locked when it completed.
				low = ADCL;
				high = ADCH;

				// combine the two bytes
				p_analog->y=(high << 8) | low;
				p_analog->estado_sensor=ESTADO_ADC_INICIANDO_CONVERSION_Z;
			}
		}
		break;

		case ESTADO_ADC_INICIANDO_CONVERSION_Z:
		{
			// start the conversion
			ADMUX = 0b01000000 | (PUERTO_Z & 0x07);
			sbi(ADCSRA, ADSC);
			p_analog->estado_sensor=ESTADO_ADC_ESPERANDO_FIN_CONVERSION_Z;
		}
		break;

		case ESTADO_ADC_ESPERANDO_FIN_CONVERSION_Z:
		{
			// ADSC is cleared when the conversion finishes
			if (bit_is_set(ADCSRA, ADSC)==0)
			{

				// we have to read ADCL first; doing so locks both ADCL
				// and ADCH until ADCH is read.  reading ADCL second would
				// cause the results of each conversion to be discarded,
				// as ADCL and ADCH would be locked when it completed.
				low = ADCL;
				high = ADCH;

				// combine the two bytes
				p_analog->z=(high << 8) | low;
				p_analog->estado=ESTADO_ANALOG_LEYENDO_PROXIMIDAD;
				p_analog->estado_sensor=ESTADO_ADC_REPOSO;
			}
		}
		break;
	}
}


void analogRead_prox(analog* p_analog)
{
	uint8_t low, high;

	switch (p_analog->estado_sensor)
	{
		case ESTADO_ADC_REPOSO:
		{
			p_analog->estado_sensor=ESTADO_ADC_INICIANDO_CONVERSION_X;
		}
		break;

		case ESTADO_ADC_INICIANDO_CONVERSION_X:
		{
			// start the conversion
			ADMUX = 0b01000000 | (PUERTO_PROX & 0x07);
			sbi(ADCSRA, ADSC);
			p_analog->estado_sensor=ESTADO_ADC_ESPERANDO_FIN_CONVERSION_X;
		}
		break;

		case ESTADO_ADC_ESPERANDO_FIN_CONVERSION_X:
		{
			// ADSC is cleared when the conversion finishes
			if (bit_is_set(ADCSRA, ADSC)==0)
			{

				// we have to read ADCL first; doing so locks both ADCL
				// and ADCH until ADCH is read.  reading ADCL second would
				// cause the results of each conversion to be discarded,
				// as ADCL and ADCH would be locked when it completed.
				low = ADCL;
				high = ADCH;

				// combine the two bytes
				p_analog->prox=(high << 8) | low;
				p_analog->estado=ESTADO_ANALOG_MUESTREO_TERMINADO;
				p_analog->estado_sensor=ESTADO_ADC_REPOSO;
			}
		}
		break;
	}
}


void inicializar_subestado_adc(analog* p_analog)
{
	p_analog->estado_sensor=ESTADO_ADC_REPOSO;
}


