/*
 * I2C.cpp
 *
 *  Created on: 07/10/2010
 *      Author: mindurain
 */

#include "I2C.h"

#include "Arduino/HardwareSerial.h"
#include "Arduino/Wire.h"


void leyendo_accel(i2c* p_i2c);
void leyendo_magneto(i2c* p_i2c);
//void leyendo_distancia(int deviceaddress, int datamode, i2c* p_i2c);

int AccelAddress = 0x53;
int CompassAddress = 0x1E;  //0x3C //0x3D;  //(0x42>>1);
int deviceaddress = 0x20;  //sensor de proximidad

int datamode = 2;  // modo de lectura del sensor de proximidad en cm.

int SENSOR_SIGN[9] = {-1,1,-1,1,1,1,-1,-1,-1};  //Correct directions x,y,z - gyros, accels, magnetormeter
int AN[6]; //array that store the 3 ADC filtered data (gyros)
int AN_OFFSET[6]={0,0,0,0,0,0}; //Array that stores the Offset of the sensors
int ACC[3];          //array that store the accelerometers data


void idosc(i2c* p_i2c)
{
	switch(p_i2c->estado)
	{
		case ESTADO_I2C_REPOSO:
		{
			//se mete el timer para cambiar el estado reposo
		}
		break;

		case ESTADO_I2C_LEYENDO_ACCEL:
		{
			leyendo_accel(p_i2c);
		}
		break;

		case ESTADO_I2C_LEYENDO_MAGNETO:
		{
			leyendo_magneto(p_i2c);
		}
		break;

		/*case ESTADO_I2C_LEYENDO_DISTANCIA:
		{
			leyendo_distancia(deviceaddress,datamode,p_i2c);
		}
		break;*/

		case ESTADO_I2C_MUESTREO_TERMINADO:
		{

		}
		break;
	}
}


static int i=0;
static char buff[6];

void leyendo_accel(i2c* p_i2c)
{
	// Reads x,y and z accelerometer registers

	switch(p_i2c->estado_accel)
	{
		case (ESTADO_SENSOR_BEGIN):
		{
			Wire.beginTransmission(AccelAddress);
			Wire.send(0x32);        //sends address to read from
			p_i2c->estado_accel=ESTADO_SENSOR_END_TRANSMISSION;
		}
		break;

		case (ESTADO_SENSOR_END_TRANSMISSION):
		{
			Wire.endTransmission2(p_i2c); //end transmission
			if(p_i2c->subestado==SUBESTADO_FIN)
			{
				p_i2c->estado_accel=ESTADO_SENSOR_RECEIVE;
				p_i2c->subestado=SUBESTADO_1;
				Wire.beginTransmission(AccelAddress); //start transmission to device
			}
		}
		break;

		case (ESTADO_SENSOR_RECEIVE):
		{
			Wire.requestFrom2(AccelAddress, 6, p_i2c);    // request 6 bytes from device
			if(p_i2c->subestado==SUBESTADO_FIN)
			{
				p_i2c->estado_accel=ESTADO_SENSOR_ESPERANDO;
				p_i2c->subestado=SUBESTADO_1;
				i=0;
			}
		}
		break;

		case (ESTADO_SENSOR_ESPERANDO):
		{
			if(Wire.available())   // ((Wire.available())&&(i<6))
			{
				buff[i] = Wire.receive();  // receive one byte
				i++;
			}else
				p_i2c->estado_accel=ESTADO_SENSOR_END_TRANSMISSION_2;
		}
		break;

		case (ESTADO_SENSOR_END_TRANSMISSION_2):
		{
			Wire.endTransmission2(p_i2c); //end transmission
			if(p_i2c->subestado==SUBESTADO_FIN)
			{
				p_i2c->estado_accel=ESTADO_SENSOR_FIN;
				p_i2c->subestado=SUBESTADO_1;
			}
		}
		break;

		case (ESTADO_SENSOR_FIN):
		{
			if (i==6)  // All bytes received?
			{
				ACC[1] = (((int)buff[1]) << 8) | buff[0];    // Y axis (internal sensor x axis)
				ACC[0] = (((int)buff[3]) << 8) | buff[2];    // X axis (internal sensor y axis)
				ACC[2] = (((int)buff[5]) << 8) | buff[4];    // Z axis
				AN[3] = ACC[0];
				AN[4] = ACC[1];
				AN[5] = ACC[2];
				p_i2c->x_accel = SENSOR_SIGN[3]*(ACC[0]-AN_OFFSET[3]);
				p_i2c->y_accel = SENSOR_SIGN[4]*(ACC[1]-AN_OFFSET[4]);
				p_i2c->z_accel = SENSOR_SIGN[5]*(ACC[2]-AN_OFFSET[5]);
				p_i2c->estado=ESTADO_I2C_LEYENDO_MAGNETO;
				p_i2c->estado_magneto=ESTADO_SENSOR_BEGIN;
			}
			else
			{
				Serial.write('E');
				Serial.write(i);
			}
		}
		break;
	}
}


void leyendo_magneto(i2c* p_i2c)
{
  //int i = 0;
  //char buff[6];
	switch(p_i2c->estado_magneto)
  	{
  		case (ESTADO_SENSOR_BEGIN):
  		{
  			Wire.beginTransmission(CompassAddress);
  			Wire.send(0x03);        //sends address to read from
  			p_i2c->estado_magneto=ESTADO_SENSOR_END_TRANSMISSION;
  		}
  		break;

  		case (ESTADO_SENSOR_END_TRANSMISSION):
		{
			Wire.endTransmission2(p_i2c); //end transmission
			if(p_i2c->subestado==SUBESTADO_FIN)
			{
				p_i2c->estado_magneto=ESTADO_SENSOR_RECEIVE;
				p_i2c->subestado=SUBESTADO_1;
				Wire.beginTransmission(CompassAddress); //start transmission to device
			}
		}
		break;

  		case (ESTADO_SENSOR_RECEIVE):
		{
			Wire.requestFrom2(CompassAddress, 6, p_i2c);    // request 6 bytes from device
			if(p_i2c->subestado==SUBESTADO_FIN)
			{
				p_i2c->estado_magneto=ESTADO_SENSOR_ESPERANDO;
				p_i2c->subestado=SUBESTADO_1;
				i=0;
			}
		}
		break;

  		case (ESTADO_SENSOR_ESPERANDO):
		{
			if(Wire.available())   // ((Wire.available())&&(i<6))
			{
				buff[i] = Wire.receive();  // receive one byte
				i++;
			}else
				p_i2c->estado_magneto=ESTADO_SENSOR_END_TRANSMISSION_2;
		}
		break;

  		case (ESTADO_SENSOR_END_TRANSMISSION_2):
		{
			Wire.endTransmission2(p_i2c); //end transmission
			if(p_i2c->subestado==SUBESTADO_FIN)
			{
				p_i2c->estado_magneto=ESTADO_SENSOR_FIN;
				p_i2c->subestado=SUBESTADO_1;
			}
		}
		break;

  		case (ESTADO_SENSOR_FIN):
		{
			if (i==6)  // All bytes received?
			{
				p_i2c->x_magneto = SENSOR_SIGN[6]*((((int)buff[2]) << 8) | buff[3]);    // X axis (internal sensor y axis)
				p_i2c->y_magneto = SENSOR_SIGN[7]*((((int)buff[0]) << 8) | buff[1]);    // Y axis (internal sensor x axis)
				p_i2c->z_magneto = SENSOR_SIGN[8]*((((int)buff[4]) << 8) | buff[5]);    // Z axis
				p_i2c->estado=ESTADO_I2C_MUESTREO_TERMINADO;
				p_i2c->estado_accel=ESTADO_SENSOR_BEGIN;
			}
			else
			{
				Serial.write('E');
				Serial.write(i);
			}
		}
  		break;
  	}
}

/*void leyendo_distancia(int deviceaddress, int datamode, i2c* p_i2c)
{
	//p_i2c->estado=ESTADO_I2C_MUESTREO_TERMINADO;
	//return;
	//Serial.write(p_i2c->estado_prox);

	switch(p_i2c->estado_prox)
	  	{
	  		case (ESTADO_SENSOR_BEGIN):
	  		{
	  			Wire.beginTransmission(deviceaddress);
	  			Wire.send(datamode);        //lo pone en modo de lectura en cm
	  			p_i2c->estado_prox=ESTADO_SENSOR_END_TRANSMISSION;
	  		}
	  		break;

	  		case (ESTADO_SENSOR_END_TRANSMISSION):
			{
				Wire.endTransmission2(p_i2c); //end transmission
				if(p_i2c->subestado==SUBESTADO_FIN)
				{
					p_i2c->estado_prox=ESTADO_SENSOR_RECEIVE;
					p_i2c->subestado=SUBESTADO_1;
					Wire.beginTransmission(deviceaddress); //start transmission to device
				}
			}
			break;

	  		case (ESTADO_SENSOR_RECEIVE):
			{
				Wire.requestFrom2(deviceaddress,1,p_i2c);    // request 1 byte from device
				if(p_i2c->subestado==SUBESTADO_FIN)
				{
					p_i2c->estado_prox=ESTADO_SENSOR_ESPERANDO;
					p_i2c->subestado=SUBESTADO_1;
					i=0;
				}
			}
			break;

	  		case (ESTADO_SENSOR_ESPERANDO):
			{
				if(Wire.available())   // ((Wire.available())&&(i<6))
				{
					buff[i] = Wire.receive();  // receive one byte
					i++;
				}else
					p_i2c->estado_prox=ESTADO_SENSOR_END_TRANSMISSION_2;
			}
			break;

			case (ESTADO_SENSOR_END_TRANSMISSION_2):
			{
				Wire.endTransmission2(p_i2c); //end transmission
				if(p_i2c->subestado==SUBESTADO_FIN)
				{
					p_i2c->estado_prox=ESTADO_SENSOR_FIN;
					p_i2c->subestado=SUBESTADO_1;
				}
			}
			break;

			case (ESTADO_SENSOR_FIN):
			{
				if (i==1)  // All bytes received?
				{
					p_i2c->distancia = buff[0];
					p_i2c->estado=ESTADO_I2C_MUESTREO_TERMINADO;
					p_i2c->estado_accel=ESTADO_SENSOR_BEGIN;
				}
				else
				{
					Serial.write('E');
					Serial.write(i);
				}
			}
			break;
	  	}
}*/


void inicializar_subestados_i2c(i2c* p_i2c)
{
	p_i2c->subestado=SUBESTADO_1;
	p_i2c->estado_accel=ESTADO_SENSOR_BEGIN;
}


void Accel_Init()
{
	Wire.beginTransmission(AccelAddress);
	Wire.send(0x2D);  // power register
	Wire.send(0x08);  // measurement mode
	Wire.endTransmission();
	_delay_ms(5);
	Wire.beginTransmission(AccelAddress);
	Wire.send(0x31);  // Data format register
	Wire.send(0x01);  // set to 10-bit & +/-4g
	Wire.endTransmission();
	_delay_ms(5);
	// Because our main loop runs at 50Hz we adjust the output data rate to 50Hz (25Hz bandwidth)
	Wire.beginTransmission(AccelAddress);
	Wire.send(0x2C);  // Rate
	Wire.send(0x09);  // set to 50Hz, normal operation
	Wire.endTransmission();
	_delay_ms(5);
}


void Compass_Init()
{
	Wire.beginTransmission(CompassAddress);
	Wire.send(0x02);
	Wire.send(0x00);   // Set continouos mode (default to 10Hz)
	Wire.endTransmission(); //end transmission
	_delay_ms(5);
	Wire.beginTransmission(CompassAddress);
	Wire.send(0x01);   // register B
	Wire.send(0x60);   // +/- 2Ga
	Wire.endTransmission(); //end transmission
	_delay_ms(5);
}


/*void Prox_Init()
{
	//Wire.beginTransmission(deviceaddress);
	//Wire.send(datamode);        //lo pone en modo de lectura en cm
	//Wire.endTransmission(); //end transmission
	//_delay_ms(5);
}*/
