/*
 * main.cpp
 *
 *  Created on: 06/10/2010
 *      Author: mindurain
 */


#include "I2C.h"
#include "ANALOG.h"
#include "TX.h"
#include "RX.h"
#include "MOTORES.h"

//#include "Arduino/HardwareSerial.h"
//#include	"Arduino/wiring_analog.h"
//#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <stdarg.h>
#include "Arduino/Wire.h"



i2c param_i2c;
analog param_analog;
int estado_tx;
rx param_rx;
motores param_motores;



void inicializar_adc()
{
	// set a2d prescale factor to 64
	// 10 MHz / 64 = 156 KHz, inside the desired 50-200 KHz range.
	// XXX: this will not work properly for other clock speeds, and
	// this code should use F_CPU to determine the prescale factor.
	sbi(ADCSRA, ADPS2);
	sbi(ADCSRA, ADPS1);
	cbi(ADCSRA, ADPS0);

	// enable a2d conversions
	sbi(ADCSRA, ADEN);
}


#define MAX_COUNT_1	2500  // 100ms = 12500 @ 8MHz con un prescaler de 64

void inicializar_timer1() //inicializo el contador 1 (compara TCNT1 con OCR1A)
{
	//desactivar salidas timers
	cbi(TCCR1A, COM1A0);
	cbi(TCCR1A, COM1A1);
	cbi(TCCR1A, COM1B0);
	cbi(TCCR1A, COM1B1);

	// put timer 1 in compare mode
	cbi(TCCR1A, WGM10);
	cbi(TCCR1A, WGM11);
	sbi(TCCR1B, WGM12);
	cbi(TCCR1B, WGM13);

	// set timer 1 prescale factor to 64
	cbi(TCCR1B, CS12);
	sbi(TCCR1B, CS11);
	sbi(TCCR1B, CS10);

	//inicializar el contador
	TCNT1 = 0x0000;
	//definir el tope de contador
	OCR1A = MAX_COUNT_1-1;
	// se pone a uno para que salte la interrupcin
	sbi(TIMSK1, OCIE1A);
}

void inicializar_timer0() //inicializo el contador 0 (cuando TCNT0 == OCR0A salta SIGNAL(TIMER0_COMPA_vect)
{
	sbi(DDRD,DDD6); // activo PD6 como output

	//activo la salida OC0A como non_inverting mode
	cbi(TCCR0A, COM0A0);
	sbi(TCCR0A, COM0A1);
	cbi(TCCR0A, COM0B0);
	cbi(TCCR0A, COM0B1);

	// put timer 0 in fast PWM mode to 0xFF (hasta contar 256 a 8MHZ es un perodo de 31,25KHz)
	sbi(TCCR0A, WGM00);
	sbi(TCCR0A, WGM01);
	cbi(TCCR0B, WGM02);

	// set timer 0 prescale factor to 1
	cbi(TCCR0B, CS02);
	cbi(TCCR0B, CS01);
	sbi(TCCR0B, CS00);

	//inicializar el contador
	TCNT0 = 0x00;
	//definir el tope de contador
	OCR0A = 0x00;
}

void inicializar_timer2() //inicializo el contador 2 (cuando TCNT2 == OCR2A salta SIGNAL(TIMER2_COMPA_vect)
													//(cuando TCNT2 == OCR2B salta SIGNAL(TIMER2_COMPB_vect)
													//(cuando TCNT2 == 0xFF salta SIGNAL (TIMER2_OVF_vect)
{
	// poner PB3 y PB4 como salidas
	sbi(DDRB,DDB3);
	sbi(DDRB,DDB4);

	//desactivar salidas timers
	cbi(TCCR2A, COM2A0);
	cbi(TCCR2A, COM2A1);
	cbi(TCCR2A, COM2B0);
	cbi(TCCR2A, COM2B1);

	// put timer 2 in normal mode
	cbi(TCCR2A, WGM20);
	cbi(TCCR2A, WGM21);
	cbi(TCCR2B, WGM22);

	// set timer 2 prescale factor to 64
	cbi(TCCR2B, CS22);
	sbi(TCCR2B, CS21);
	sbi(TCCR2B, CS20);

	//inicializar el contador
	TCNT2 = 0x00;
	//definir el tope de contador
	OCR2A = 0x00;
	OCR2B = 0X00;

	// se pone a uno para que salten las interrupciones
	sbi(TIMSK2, OCIE2A);
	sbi(TIMSK2, OCIE2B);
	sbi(TIMSK2, TOIE2);
}

void inicializar_procesos()
{
	param_i2c.estado=ESTADO_I2C_REPOSO;
	param_analog.estado=ESTADO_ANALOG_REPOSO;
	estado_tx=ESTADO_TX_REPOSO;
	param_rx.estado=ESTADO_RX_REPOSO;
	param_motores.estado_motores=ESTADO_MOTORES_REPOSO;
}



void setup()
{
	sbi(DDRB,DDB5); // poner como PB5 como salida
	UCSR0B = 0;
	//UCSR1B = 0;
	Serial.begin(57600);
	Wire.begin();        // join i2c bus (address optional for master)
	inicializar_timer1();
	inicializar_timer0();
	inicializar_timer2();
	sei();
	inicializar_adc();
	inicializar_procesos(); // incializamos todos los procesos al estado reposo
	inicializar_subestado_adc(&param_analog);
	inicializar_subestados_i2c(&param_i2c);
	//_delay_ms(100);
	Compass_Init();
	Accel_Init();
	//Prox_Init();
}

extern "C" void __cxa_pure_virtual() //necesario para usar libreras de arduino
{
	for (;;);
}

int main()
{
	setup();

	for(;;)
	{
		idosc(&param_i2c);
		analg(&param_analog);
		tx(&estado_tx, &param_i2c, &param_analog);
		Rx(&param_rx);
		motors(&param_motores,&param_rx);
	}
}


SIGNAL(TIMER1_COMPA_vect) // se ejecuta ,independientemente de la lnea de programa del main, cada 100 ms;
{
	if ((param_i2c.estado==ESTADO_I2C_REPOSO) && (param_analog.estado==ESTADO_ANALOG_REPOSO)) //comenzara un nuevo muestreo si se ha ejecutado todo el proceso i2c
	{
		param_i2c.estado=ESTADO_I2C_LEYENDO_ACCEL;
		param_analog.estado=ESTADO_ANALOG_LEYENDO_GYRO;
	}
}

SIGNAL(TIMER2_COMPA_vect)
{
	cbi(PORTB,PORTB3);
}

SIGNAL(TIMER2_COMPB_vect)
{
	cbi(PORTB,PORTB4);
}

SIGNAL (TIMER2_OVF_vect)
{
	if (OCR2A>0x0F)
		sbi(PORTB,PORTB3);
	if (OCR2B>0x0F)
		sbi(PORTB,PORTB4);
}
